import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.List;
public class Enemy extends SmoothMover implements Moveable, Killable
{ 
    public int dX;
    public int dY;
    public int Hp;
    public double speed;
    public double pushX;
    public double pushY;
    public int r;
    public double mass;
    public Enemy()
    {
        dX = 0;
        dY = 0;
        Hp = 0;
        speed = 0.0;
        pushX = 0;
        pushY = 0;
        r = 0;
        mass = 0.0;
    }
    public void die()
    {
        World w = getWorld();
        w.removeObject(this);
    }
    public void takeDamage(int damage)
    {
        Hp -= damage;
    }
    public boolean atDestination()
    {
        return (((int)currentX <= dX + speed && (int)currentX >= dX - speed) && ((int)currentY <= dY + speed && (int)currentY >= dY - speed));
    }
    public void addedToWorld(World world)
    {
        currentX = getX();  
        currentY = getY();  
    }
    public void move()
    {
        if(!atDestination())
        {
            Field w = (Field)getWorld();
            move(speed);
            if(currentX-w.xPos > w.getImage().getWidth()-r)
            {
                currentX -= speed;
            }
            else if(currentX-w.xPos <= r)
            {
                currentX += speed;
            }
            if(currentY-w.yPos > w.getImage().getHeight()-r)
            {
                currentY -= speed;
            }
            else if(currentY-w.yPos <= r)
            {
                currentY += speed;
            }
            setLocation((int)currentX, (int)currentY);
        }
    }
    public void determinePushVectors()
    {
        pushX = 0;
        pushY = 0;
        List<Moveable> list = getIntersectingObjects(Moveable.class);
        int size = list.size();
        for(int n = 0; n < size; n ++)
        {
            Moveable a = list.get(n);
            double x2 = a.getCurrentX();
            double y2 = a.getCurrentY();
            
            double d = Math.sqrt((x2 - currentX)*(x2 - currentX) + (y2 - currentY)*(y2 - currentY));
            if(d < r + a.getRadius())//distance between center is less then their radius combined
            {
                //get angle up date push x and y
                double angle = Math.atan2((y2-currentY), (x2-currentX));
                double move = (r + a.getRadius()) - d;//gets how much the circles intersect
                if(a.canBePushed())
                {
                    pushX += Math.cos(angle)*(move/((mass + a.getMass())/a.getMass()));
                    pushY += Math.sin(angle)*(move/((mass + a.getMass())/a.getMass()));
                }
                else
                {
                    pushX += Math.cos(angle)*(move);
                    pushY += Math.sin(angle)*(move);
                }
            }
        }
    }
    public void displace()
    {
        Field w = (Field)getWorld();
        currentY -= pushY;
        currentX -= pushX;
        setLocation((int)currentX, (int)currentY);
        if(currentX-w.xPos <= r || currentX-w.xPos > w.getImage().getWidth()-r)// x cord is out of bounds
        {
            currentX += pushX;
        }
        if(currentY-w.yPos <= r || currentY-w.yPos > w.getImage().getHeight()-r)// y cord is out of bounds
        {
            currentY += pushY;
        }
        setLocation((int)currentX, (int)currentY);
    }
    public int getRadius()
    {
        return r;
    }
    public double getCurrentX()
    {
        return currentX;
    }
    public double getCurrentY()
    {
        return currentY;
    }
    public void setCord(int x, int y)
    {
        dX += x;
        dY += y;
    }
    public double getMass()
    {
        return mass;
    }
    public boolean canBePushed()
    {
        return true;
    }
}
